package com.googlecode.mad.mvntools.settingsutil.filestore;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import com.googlecode.mad.mvntools.common.Factory;
import com.googlecode.mad.mvntools.common.interfaces.IFileUtil;
import com.googlecode.mad.mvntools.common.interfaces.IMavenVersionUtil;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * - Apache License 2.0 - <br>
 * Abstract Mojo to backup and restore a .x files from a project <br>
 * <br>
 * LastChangedBy: $Author: marthaler.worb@gmail.com $<br>
 * LastChangedDate: $Date: 2010-12-29 14:14:06 +0100 (Mi, 29 Dez 2010) $
 * 
 * @author daniel marthaler
 * @version $Revision: 21 $
 * @since 24.12.2010
 */
public abstract class AbstractFilestore extends AbstractMojo {
    
    protected final static String                 FOLDER        = "XXfolder:";
    protected final static String                 FILE          = "XXfile:";
    protected final static String                 USER          = "default";
    protected final static String                 USER_KEY      = "user.name";
    protected final static String                 SEPARATOR_KEY = "file.separator";
    protected final static String                 TARGET        = "target";
    protected final static String                 EMTYSTRING    = "";
    
    protected String                              fileName      = "abstractFilestore";
    
    protected XStream                             xstream       = new XStream(new DomDriver());
    
    /**
     * The project dependencies. *
     * 
     * @parameter expression="${project.build.directory} "
     */
    protected String                              buildDir;
    
    protected String[]                            files         = { ".project", ".classpath" };
    
    protected String[]                            folders       = { ".settings" };
    
    /**
     * The configured files
     * 
     * @parameter alias="files"
     */
    private String[]                              providedFiles;
    
    /**
     * The configured folders
     * 
     * @parameter alias="folders"
     */
    private String[]                              providedFolders;
    
    /**
     * The name of the backup-file
     * 
     * @parameter alias="backupfile" default-value="settingsutil.xml"
     */
    protected String                              backupFile;
    
    /**
     * The User
     * 
     * @parameter expression="${name}" default-value="default"
     */
    protected String                              name;
    
    /**
     * @parameter default-value="${project}"
     */
    private org.apache.maven.project.MavenProject mavenProject;
    
    protected String                              delimiter;
    
    protected IFileUtil                           fileUtil      = Factory.getInstance().getIFileUtil();
    
    protected IMavenVersionUtil                   mvnUtil       = Factory.getInstance().getIMavenVersionUtil();
    
    public void execute() throws MojoExecutionException {
        
        this.getLog().info(this.mvnUtil.getMavenString(this.mavenProject.getGroupId(), this.mavenProject.getArtifactId(), this.getClass()));
        
        // overwrite the default files and folders if there are cnfigured ones
        if ((this.providedFiles != null) && (this.providedFiles.length > 0)) {
            this.files = this.providedFiles;
        }
        
        if ((this.providedFolders != null) && (this.providedFolders.length > 0)) {
            this.folders = this.providedFolders;
        }
        // set the user
        if (this.name.equals(AbstractFilestore.USER)) {
            this.name = System.getProperty(AbstractFilestore.USER_KEY);
        }
        // set the delimiter
        this.delimiter = System.getProperty(AbstractFilestore.SEPARATOR_KEY);
        
        this.myExecute();
        
    }
    
    protected abstract void myExecute();
    
    protected String getRootPath() {
        return this.buildDir.replace(AbstractFilestore.TARGET, AbstractFilestore.EMTYSTRING);
    }
    
}
